/*************************************************************************/
/*************************************************************************/
/******                                                             ******/
/******  DATE / TIME UTILITIES                                      ******/
/******                                                             ******/
/*************************************************************************/
/*************************************************************************/

/* Revision history:
     91.12.01 Created, translated from Pascal.
     92.09.22 DateSep added as variable, allow '.' in dates.
*/

#include <time.h>
#include <sys/types.h>

char DayName[ 7 ][ 20 ] =
      { "Sunday", "Monday", "Tuesday", "Wednesday",
        "Thursday", "Friday", "Saturday" };

char MonthName[ 12 ][ 20 ] =
      { "January", "February", "March", "April", "May", "June", "July",
        "August", "September", "October", "November", "December" };

int DaysPerMonth[ 12 ] =
      { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

/* DateFormat specifies the format of dates for input (Inp_date) */
/* and output (ShowDate).  Internally all dates YYMMDD. */
/* Formats are: 0=YMD 1=MDY 2=DMY */
int DateFormat = 0;

/* Separator between sections of date. */
char DateSep = '.';

/*-----------------------------------------------------------------------*/
/* <ParseDate> breaks a date in any reasonable string format into        */
/* separate Yr, Mo, Da integers.  Returns false if cannot parse.         */
/* Valid formats include YYMD, YYMDD, YYMMDD, YY/M/D, YY-MM-DD, ...      */
/*-----------------------------------------------------------------------*/

int ParseDate( DateStr, Yr, Mo, Da )
char *DateStr;
int *Yr, *Mo, *Da;
{
  char PrevCh;
  char WorkDateStr[9];
  char YrStr[3], MoStr[3], DaStr[3];
  int ParseLoop, WorkLoop, MarkCnt, Mark1Pos, Mark2Pos, PrevMark;

  /* initialize */
  *Yr = -1;  *Mo = -1;  *Da = -1;
  /* Length must be between 4 (YYMD) and 8 (YY/MM/DD) */
  ParseLoop = 0;
  while ( ( ParseLoop <= 8 ) && ( DateStr[ ParseLoop ] ) )
    ParseLoop++;
  if ( ( ParseLoop < 4 ) || ( ParseLoop == 9 ) )
    return( 0 );
  /* remove "/" or "-", confirm none adjacent, misplaced, or extra */
  Mark1Pos = 0;  Mark2Pos = 0;
  PrevMark = 1;  MarkCnt = 0;
  memset( WorkDateStr, '\0', sizeof( WorkDateStr ) );
  WorkLoop = 0;
  for ( ParseLoop = 0; DateStr[ ParseLoop ]; ParseLoop++ )
  {
    if ( ( DateStr[ ParseLoop ] >= '0' ) &&
         ( DateStr[ ParseLoop ] <= '9' ) )
    {
      /* valid character */
      WorkDateStr[ WorkLoop++ ] = DateStr[ ParseLoop ];
      PrevMark = 0;
    }
    else
    if ( ( DateStr[ ParseLoop ] == '/' ) ||
         ( DateStr[ ParseLoop ] == '-' ) ||
         ( DateStr[ ParseLoop ] == '.' ) )
    {
      /* check for marks adjacent, too soon or too many */
      if ( ( PrevMark ) ||
           ( strlen( WorkDateStr ) < 2 ) ||
           ( MarkCnt >= 2 ) )
        return( 0 );
      MarkCnt++;
      if ( Mark1Pos )
        Mark2Pos = WorkLoop;
      else
        Mark1Pos = WorkLoop;
      PrevMark = 1;
    }
    else
      /* invalid character */
      return( 0 );
  }
  if ( PrevMark ) return( 0 );
  if ( ! Mark1Pos ) Mark1Pos = 2;
  if ( ! Mark2Pos )
  {  /* YMD */
    if ( strlen( WorkDateStr ) < 6 )
      Mark2Pos = 3;
    else
      Mark2Pos = 4;
  }
  /* too far apart? */
  if ( Mark2Pos - Mark1Pos > 2 )
    return( 0 );
  /* break into substrings */
  *YrStr = '\0';  *MoStr = '\0';  *DaStr = '\0';
  strncpy( YrStr, &WorkDateStr[ 0 ], Mark1Pos );
  YrStr[ Mark1Pos ] = '\0';
  strncpy( MoStr, &WorkDateStr[ Mark1Pos ], Mark2Pos - Mark1Pos );
  MoStr[ Mark2Pos - Mark1Pos ] = '\0';
  strncpy( DaStr, &WorkDateStr[ Mark2Pos ],
                  strlen( WorkDateStr ) - Mark2Pos );
  DaStr[ strlen( WorkDateStr ) - Mark2Pos ] = '\0';
  /* convert to numeric */
  if ( ! *YrStr )
    *Yr = 70;
  else
    sscanf( YrStr, "%d", Yr );
  if ( *Yr < 70 ) *Yr += 100;  /* assume years < 70 in 21st century */
  *Yr += 1900;
  if ( ! *MoStr )
    *Mo = 1;
  else
    sscanf( MoStr, "%d", Mo );
  if ( ! *DaStr )
    *Da = 1;
  else 
    sscanf( DaStr, "%d", Da );
  return( 1 );
}

/*-----------------------------------------------------------------------*/
/* <ValidYMD> returns true if year/month/day makes valid date.           */
/*-----------------------------------------------------------------------*/

int ValidYMD( Yr, Mo, Da )
int Yr, Mo, Da;
{
  /* check for obvious out of range */
  if ( ( Yr >= 1970 ) && ( Yr <= 2069 ) ) Yr %= 100;
  if ( ( Yr < 0 ) || ( Yr > 99 ) ||
       ( Mo < 1 ) || ( Mo > 12 ) ||
       ( Da < 1 ) || ( Da > 31 ) )
    return( 0 );
  /* check for valid day in month */
  if ( ( Da > DaysPerMonth[ Mo - 1 ] ) &&
       ( ! ( ( Da == 29 ) && ( Mo == 2 ) && ( Yr % 4 == 0 ) ) ) )
    return( 0 );
  return( 1 );
}

/*-----------------------------------------------------------------------*/
/* <ValidDate> returns true if date string is valid date.                */
/*-----------------------------------------------------------------------*/

int ValidDate( DateStr )
char *DateStr;
{
  int Yr, Mo, Da;

  if ( ! ParseDate( DateStr, &Yr, &Mo, &Da ) )
    return( 0 );
  if ( ! ValidYMD( Yr, Mo, Da ) )
    return( 0 );
  return( 1 );
}

/*-----------------------------------------------------------------------*/
/* <ParseTime> breaks a time in any reasonable string format into        */
/* separate Hr, Mn, Sc integers.  Returns false if cannot parse.         */
/* Valid formats include H:M:S, HH:MM, HH:MMam, H:MM:SS, Hpm, ...        */
/*-----------------------------------------------------------------------*/

int ParseTime( TimeStrIn, Hr, Mn, Sc )
char *TimeStrIn;
int *Hr, *Mn, *Sc;
{
  char PrevCh;
  char WorkTimeStr[9];
  char TimeStr[11];
  char HrStr[3], MnStr[3], ScStr[3], OfsStr[3];
  int ParseLoop, WorkLoop, MarkCnt, HrOfs, Mark1Pos, Mark2Pos, PrevMark;

  /* initialize */
  *Hr = -1;  *Mn = -1;  *Sc = -1;
  strncpy( TimeStr, TimeStrIn, sizeof( TimeStr ) );
  /* Length must be between 1 (H) and 10 (HH:MM:SSpm) */
  ParseLoop = 0;
  while ( ( ParseLoop <= 10 ) && ( TimeStr[ ParseLoop ] ) )
    ParseLoop++;
  if ( ( ParseLoop < 1 ) || ( ParseLoop == 11 ) )
    return( 0 );
  /* check for am/pm */
  HrOfs = -1;
  if ( strlen( TimeStr ) >= 3 )
  {
    strncpy( OfsStr, &TimeStr[ strlen( TimeStr ) - 2 ], 2 );
    OfsStr[ 2 ] = '\0';
    if ( ( strcmp( OfsStr, "am" ) == 0 ) ||
         ( strcmp( OfsStr, "AM" ) == 0 ) )
      HrOfs = 0;
    else
    if ( ( strcmp( OfsStr, "pm" ) == 0 ) ||
         ( strcmp( OfsStr, "PM" ) == 0 ) )
      HrOfs = 12;
    if ( HrOfs > -1 )  /* truncate */
      TimeStr[ strlen( TimeStr ) - 2 ] = '\0';
  }
  /* check for a/p */
  if ( ( HrOfs == -1 ) && ( strlen( TimeStr ) >= 2 ) )
  {
    strncpy( OfsStr, &TimeStr[ strlen( TimeStr ) - 1 ], 1 );
    OfsStr[ 1 ] = '\0';
    if ( ( strcmp( OfsStr, "a" ) == 0 ) ||
         ( strcmp( OfsStr, "A" ) == 0 ) )
      HrOfs = 0;
    else
    if ( ( strcmp( OfsStr, "p" ) == 0 ) ||
         ( strcmp( OfsStr, "P" ) == 0 ) )
      HrOfs = 12;
    if ( HrOfs > -1 )  /* truncate */
      TimeStr[ strlen( TimeStr ) - 1 ] = '\0';
  }
  /* remove ":", confirm none adjacent, misplaced, or extra */
  Mark1Pos = 0;  Mark2Pos = 0;
  PrevMark = 1;  MarkCnt = 0;
  memset( WorkTimeStr, '\0', sizeof( WorkTimeStr ) );
  WorkLoop = 0;
  for ( ParseLoop = 0; TimeStr[ ParseLoop ]; ParseLoop++ )
  {
    if ( ( TimeStr[ ParseLoop ] >= '0' ) &&
         ( TimeStr[ ParseLoop ] <= '9' ) )
    {
      /* valid character */
      WorkTimeStr[ WorkLoop++ ] = TimeStr[ ParseLoop ];
      PrevMark = 0;
    }
    else
    if ( TimeStr[ ParseLoop ] == ':' )
    {
      /* check for marks adjacent, too soon or too many */
      if ( ( PrevMark ) ||
           ( ! *WorkTimeStr ) ||
           ( MarkCnt >= 2 ) )
        return( 0 );
      MarkCnt++;
      if ( Mark1Pos )
        Mark2Pos = WorkLoop;
      else
        Mark1Pos = WorkLoop;
      PrevMark = 1;
    }
    else
      /* invalid character */
      return( 0 );
  }
  if ( PrevMark )
    return( 0 );
  if ( ! Mark1Pos )
  {   /* H HH HMM HHMM HMMSS HHMMSS */
    switch ( strlen( WorkTimeStr ) )
    {
      case 1:  Mark1Pos = 1;  Mark2Pos = 1;  break;
      case 2:  Mark1Pos = 2;  Mark2Pos = 2;  break;
      case 3:  Mark1Pos = 1;  Mark2Pos = 3;  break;
      case 4:  Mark1Pos = 2;  Mark2Pos = 4;  break;
      case 5:  Mark1Pos = 1;  Mark2Pos = 3;  break;
      case 6:  Mark1Pos = 2;  Mark2Pos = 4;  break;
    }
  }
  if ( ! Mark2Pos )
  {
    if ( Mark1Pos + 2 > strlen( WorkTimeStr ) )
      Mark2Pos = strlen( WorkTimeStr );
    else
      Mark2Pos = Mark1Pos + 2;
  }
  /* too far apart? */
  if ( Mark2Pos - Mark1Pos > 2 )
    return( 0 );
  /* break into substrings */
  *HrStr = '\0';  *MnStr = '\0';  *ScStr = '\0';
  strncpy( HrStr, &WorkTimeStr[ 0 ], Mark1Pos );
  HrStr[ Mark1Pos ] = '\0';
  strncpy( MnStr, &WorkTimeStr[ Mark1Pos ], Mark2Pos - Mark1Pos );
  MnStr[ Mark2Pos - Mark1Pos ] = '\0';
  strncpy( ScStr, &WorkTimeStr[ Mark2Pos ],
                  strlen( WorkTimeStr ) - Mark2Pos );
  ScStr[ strlen( WorkTimeStr ) - Mark2Pos ] ='\0';
  /* convert to numeric */
  if ( ! *HrStr )
    *Hr = 0;
  else
    sscanf( HrStr, "%d", Hr );
  if ( HrOfs > 0 ) *Hr = *Hr + 12;
  if ( ! *MnStr )
    *Mn = 0;
  else
    sscanf( MnStr, "%d", Mn );
  if ( ! *ScStr )
    *Sc = 0;
  else
    sscanf( ScStr, "%d", Sc );
  return( 1 );
}

/*-----------------------------------------------------------------------*/
/* <ValidHMS> returns true if hours/minutes/seconds make valid time.     */
/*-----------------------------------------------------------------------*/

int ValidHMS( Hr, Mn, Sc )
int Hr, Mn, Sc;
{
  /* check for range */
  return( ( Hr >= 0 ) && ( Hr <= 23 ) &&
          ( Mn >= 0 ) && ( Mn <= 59 ) &&
          ( Sc >= 0 ) && ( Sc <= 59 ) );
}

/*-----------------------------------------------------------------------*/
/* <ValidTime> returns true if time string is valid time.                */
/*-----------------------------------------------------------------------*/

int ValidTime( TimeStr )
char *TimeStr;
{
  int Hr, Mn, Sc;

  if ( ! ParseTime( TimeStr, &Hr, &Mn, &Sc ) )
    return( 0 );
  if ( ! ValidHMS( Hr, Mn, Sc ) )
    return( 0 );
  return( 1 );
}

/*-----------------------------------------------------------------------*/
/* <DTSec> returns seconds since 00:00:00 70/01/01                       */
/* as a real number, given date and time strings YY/MM/DD and HH:MM:SS.  */
/* Returns -1.0 for invalid date or time.                                */
/*-----------------------------------------------------------------------*/

double DTSec( DateStr, TimeStr )
char *DateStr, *TimeStr;
{
  int Yr, Mo, Da, Hr, Mn, Sc;
  long DaTotal, ScTotal;
  int DTLp;

  /* check validity */
  if ( ! ParseDate( DateStr, &Yr, &Mo, &Da ) )
    return( -1.0 );
  if ( ! ValidYMD( Yr, Mo, Da ) )
    return( -1.0 );
  if ( ! ParseTime( TimeStr, &Hr, &Mn, &Sc ) )
    return( -1.0 );
  if ( ! ValidHMS( Hr, Mn, Sc ) )
    return( -1.0 );
  /* convert date to days since 01-01-70 */
  Yr = Yr - 1970;
  /* start with leap year days */
  DaTotal = Yr / 4;
  /* compensate if current year is leap year, but leap day is not past */
  if ( ( Yr % 4 == 0 ) &&
       ( ( Mo < 2 ) || ( ( Mo == 2 ) && ( Da < 29 ) ) ) )
    DaTotal--;
  DaTotal = DaTotal + ( Yr * 365 );
  DTLp = 1;
  /* add days for elapsed months in year */
  while ( DTLp < Mo )
  {
    DaTotal = DaTotal + DaysPerMonth[ DTLp - 1 ];
    DTLp++;
  }
  /* add elapsed days in month to finish */
  DaTotal += Da;
  /* convert time to seconds since 00:00:00 */
  ScTotal = Hr * 3600 + Mn * 60 + Sc;
  return( (double)DaTotal * 86400.0 + ScTotal * 1.0 );
}

/*-----------------------------------------------------------------------*/
/* <SecDT> returns a string containing date and time "YY/MM/DD HH:MM:SS" */
/* given a real number of seconds since 00:00:00 70/01/01.               */
/*-----------------------------------------------------------------------*/

char *SecDT( InSec )
double InSec;
{
  static char SecDTOut[ 18 ];
  long Yr, Mo, Da, Hr, Mn, Sc, DaTotal, ScTotal;
  int DaysInCurrMonth;

  *SecDTOut = '\0';
  DaTotal = InSec / 86400.0 + 1;
  ScTotal = InSec - ( DaTotal - 1 ) * 86400.0;
  Hr = ScTotal / 3600;
  Mn = ( ScTotal - ( Hr * 3600 ) ) / 60;
  Sc = ScTotal - ( Hr * 3600 ) - ( Mn * 60 );
  Yr = 1970;  Mo = 1;  Da = 1;
  if ( ( Mo != 2 ) || ( Yr % 4 != 0 ) )
    DaysInCurrMonth = DaysPerMonth[ Mo - 1 ];
  else
    DaysInCurrMonth = 29;
  while ( DaTotal > DaysInCurrMonth )
  {
    DaTotal = DaTotal - DaysInCurrMonth;
    Mo++;
    if ( Mo > 12 )
    {
      Mo = 1;
      Yr++;
    }
    if ( ( Mo != 2 ) || ( Yr % 4 != 0 ) )
      DaysInCurrMonth = DaysPerMonth[ Mo - 1 ];
    else
      DaysInCurrMonth = 29;
  }
  Da = DaTotal;
  SecDTOut[ 0 ] = Yr % 100 / 10 + '0';
  SecDTOut[ 1 ] = Yr % 10 + '0';
  SecDTOut[ 2 ] = DateSep;
  SecDTOut[ 3 ] = Mo / 10 + '0';
  SecDTOut[ 4 ] = Mo % 10 + '0';
  SecDTOut[ 5 ] = DateSep;
  SecDTOut[ 6 ] = Da / 10 + '0';
  SecDTOut[ 7 ] = Da % 10 + '0';
  SecDTOut[ 8 ] = ' ';
  SecDTOut[ 9 ] = Hr / 10 + '0';
  SecDTOut[ 10 ] = Hr % 10 + '0';
  SecDTOut[ 11 ] = ':';
  SecDTOut[ 12 ] = Mn / 10 + '0';
  SecDTOut[ 13 ] = Mn % 10 + '0';
  SecDTOut[ 14 ] = ':';
  SecDTOut[ 15 ] = Sc / 10 + '0';
  SecDTOut[ 16 ] = Sc % 10 + '0';
  SecDTOut[ 17 ] = '\0';
  return( SecDTOut );
}

/*-----------------------------------------------------------------------*/
/* <Sec_tm> fills a standard tm structure given a real number            */
/* of seconds since 00:00:00 70/01/01.                                   */
/*-----------------------------------------------------------------------*/

int Sec_tm( tm_rec, InSec )
struct tm *tm_rec;
double InSec;
{
  long Yr, Mo, Da, DoY, Hr, Mn, Sc, DaTotal, ScTotal;
  int DaysInCurrMonth;

  memset( tm_rec, 0, sizeof( *tm_rec ) );
  if ( InSec < 0.0 ) return( 0 );
  DaTotal = InSec / 86400.0 + 1;
  ScTotal = InSec - ( DaTotal - 1 ) * 86400.0;
  Hr = ScTotal / 3600;
  Mn = ( ScTotal - ( Hr * 3600 ) ) / 60;
  Sc = ScTotal - ( Hr * 3600 ) - ( Mn * 60 );
  Yr = 1970;  Mo = 1;  Da = DaTotal;  DoY = 0;
  DaysInCurrMonth = DaysPerMonth[ 0 ];
  /* subtracting days, incrementing month and year */
  while ( Da > DaysInCurrMonth )
  {
    Da -= DaysInCurrMonth;
    DoY += DaysInCurrMonth;
    Mo++;
    if ( Mo > 12 )
    {
      Mo = 1;
      Yr++;
      DoY = 0;
    }
    if ( ( Mo != 2 ) || ( Yr % 4 != 0 ) )
      DaysInCurrMonth = DaysPerMonth[ Mo - 1 ];
    else
      DaysInCurrMonth = 29;
  }
  DoY += Da;
  tm_rec->tm_year = Yr - 1900;
  tm_rec->tm_mon = Mo - 1;
  tm_rec->tm_mday = Da;
  tm_rec->tm_hour = Hr;
  tm_rec->tm_min = Mn;
  tm_rec->tm_sec = Sc;
  tm_rec->tm_wday = ( DaTotal + 3 ) % 7;
  tm_rec->tm_yday = DoY - 1;
  tm_rec->tm_isdst = 0;  /* cannot calculate */
  return( 1 );
}

/*-----------------------------------------------------------------------*/
/* <str2> returns a two character length string for an integer.          */
/*-----------------------------------------------------------------------*/

char *str2( InNum )
int InNum;
{
  static char str2_out[3];

  InNum = InNum % 100;
  if ( InNum < 10 )
    sprintf( str2_out, "0%d", InNum );
  else
    sprintf( str2_out, "%d", InNum );
  return( str2_out );
}

/*-----------------------------------------------------------------------*/
/* <ShowDate> formats a date string into some basic styles:              */
/*  0: YY/MM/DD  1: Tue., Jan. 1, 1990  2: Tuesday, January 1st., 1990   */
/* Note that format 0 is controlled by DateFormat variable.              */
/*-----------------------------------------------------------------------*/

char *ShowDate( InDate, Style )
char *InDate;
int Style;
{
  static char ShowDateOut[ 60 ];
  struct tm tm_rec;
  double DT_d;
  int Yr, Mo, Da;
  char YrStr[ 15 ], MoStr[ 15 ], DOWStr[ 15 ], DaStr[ 15 ];

  strcpy( ShowDateOut, "invalid date" );
  if ( ! ParseDate( InDate, &Yr, &Mo, &Da ) )
    return( ShowDateOut );
  if ( ! ValidYMD( Yr, Mo, Da ) )
    return( ShowDateOut );
  /* format into string */
  if ( ! Style )
  {
    strcpy( YrStr, str2( Yr ) );
    strcpy( MoStr, str2( Mo ) );
    strcpy( DaStr, str2( Da ) );
    if ( DateFormat == 1 )  /* MDY */
      sprintf( ShowDateOut, "%s%c%s%c%s",
               MoStr, DateSep, DaStr, DateSep, YrStr );
    else
    if ( DateFormat == 2 )  /* DMY */
      sprintf( ShowDateOut, "%s%c%s%c%s",
               DaStr, DateSep, MoStr, DateSep, YrStr );
    else  /* YMD */
      sprintf( ShowDateOut, "%s%c%s%c%s",
               YrStr, DateSep, MoStr, DateSep, DaStr );
  }
  else
  {
    DT_d = DTSec( InDate, "01:00:00" );
    Sec_tm( &tm_rec, DT_d );
    sprintf( DaStr, "%d", Da );
    sprintf( YrStr, "%d", Yr );
    if ( Style == 1 )
    {
      strncpy( DOWStr, DayName[ tm_rec.tm_wday ], 3 );
      DOWStr[ 3 ] = '\0';
      strcat( DOWStr, "." );
      strncpy( MoStr, MonthName[ Mo - 1 ], 3 );
      MoStr[ 3 ] = '\0';
      strcat( MoStr, "." );
    }
    else
    {
      strcpy( DOWStr, DayName[ tm_rec.tm_wday ] );
      strcpy( MoStr, MonthName[ Mo - 1 ] );
      Da = Da % 10;
      if ( Da == 1 )
        strcat( DaStr, "st." );
      else
      if ( Da == 2 )
        strcat( DaStr, "nd." );
      else
      if ( Da == 3 )
        strcat( DaStr, "rd." );
      else
        strcat( DaStr, "th." );
    }
    sprintf( ShowDateOut, "%s, %s %s, %s", DOWStr, MoStr, DaStr, YrStr );
  }
  return( ShowDateOut );
}

/*-----------------------------------------------------------------------*/
/* <Delay> pauses for a number of milliseconds, as close as allowed      */
/* by system clock resolution.                                           */
/*-----------------------------------------------------------------------*/

/* clock() and CLK_TCK are ANSI but missing from AT&T.  Cannot substitute.
int Dur;
{
  clock_t StartTick, EndTick, CurrTick;

  StartTick = clock();
  EndTick = StartTick + ( Dur * CLK_TCK / 1000 );
  do
    CurrTick = clock();
  while ( ( CurrTick < EndTick ) && ( CurrTick >= StartTick ) );
  return( 0 );
}
*/

/*-----------------------------------------------------------------------*/
/* <DateString> returns system clock date formatted as YY/MM/DD          */
/*-----------------------------------------------------------------------*/

char *DateString()
{
  static char DateStringOut[ 9 ];
  time_t CurrTime;
  char YrStr[ 3 ], MoStr[ 3 ], DaStr[ 3 ];
  struct tm tm_rec;

  time( &CurrTime );
  memcpy( &tm_rec, localtime( &CurrTime ), sizeof( tm_rec ) );
  strcpy( YrStr, str2( tm_rec.tm_year ) );
  strcpy( MoStr, str2( tm_rec.tm_mon + 1 ) );
  strcpy( DaStr, str2( tm_rec.tm_mday ) );
  sprintf( DateStringOut, "%s%c%s%c%s",
                YrStr, DateSep, MoStr, DateSep, DaStr );
  return( DateStringOut );
}

/*-----------------------------------------------------------------------*/
/* <TimeString> returns system clock time formatted as HH:MM:SS          */
/*-----------------------------------------------------------------------*/

char *TimeString()
{
  static char TimeStringOut[ 9 ];
  time_t CurrTime;
  char HrStr[ 3 ], MnStr[ 3 ], ScStr[ 3 ];
  struct tm tm_rec;

  time( &CurrTime );
  memcpy( &tm_rec, localtime( &CurrTime ), sizeof( tm_rec ) );
  strcpy( HrStr, str2( tm_rec.tm_hour ) );
  strcpy( MnStr, str2( tm_rec.tm_min ) );
  strcpy( ScStr, str2( tm_rec.tm_sec ) );
  sprintf( TimeStringOut, "%s:%s:%s", HrStr, MnStr, ScStr );
  return( TimeStringOut );
}
