/*************************************************************************/
/*************************************************************************/
/******                                                             ******/
/******  USER INTERFACE UTILITIES                                   ******/
/******                                                             ******/
/*************************************************************************/
/*************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <termio.h>

#include "aibdate.h"
#include "aibfile.h"
#include "aibmisc.h"
#include "aibstr.h"

#define EscKey 27

/* Escape is set by CheckKey, WaitKey, SelectCh and Inp* if escape */
/* is pressed.  This value must be reset in the application. */
int Escape = 0;

/*-----------------------------------------------------------------------*/
/* <Beep> simply outputs a ^G (bell) character to stdout.                */
/*-----------------------------------------------------------------------*/

int Beep()
{
  printf( "\007" );
  return( 7 );
}

/*-----------------------------------------------------------------------*/
/* <CpyRtMsg> decodes and returns encrypted copyright notice.            */
/*-----------------------------------------------------------------------*/

char CpyRtMsgStr[ 26 ] =
  { 168, 155, 102, 119, 161, 173, 118, 142, 115, 146, 41, 162, 163,
    127, 138, 129, 49, 141, 115, 163, 165, 129, 122, 126, 134, 136 };

char *CpyRtMsg()
{
  static char CpyRtOut[ 27 ];
  char CpyRtLp = 1;

  for ( CpyRtLp = 0; CpyRtLp < 26; CpyRtLp++ )
    if ( ! CpyRtLp )
      CpyRtOut[ CpyRtLp ] = CpyRtMsgStr[ CpyRtLp ] - 128;
    else
      CpyRtOut[ CpyRtLp ] =
        ( CpyRtMsgStr[ CpyRtLp ] - 128 ) + CpyRtOut[ CpyRtLp - 1 ];
  CpyRtOut[ CpyRtLp ] = '\0';
  return( CpyRtOut );
}

/*-----------------------------------------------------------------------*/
/* <ShowPercent> prints a % done figure given a real number from 0 to 1. */
/* The display updates only if input increases by .01 or more.           */
/* It must always be initialized first by passing 0.0 or less.           */
/*-----------------------------------------------------------------------*/

int ShowPercent( InPct )
float InPct;
{
  static float PrevPct = 0.0;
  int ShowPct;

  if ( InPct <= 0.0000001 )
    PrevPct = 0.0;
  else
  {
    if ( InPct - PrevPct < 0.01 ) return( 0 );
    printf( "\b\b\b\b\b\b\b\b\b" );
  }
  ShowPct = InPct * 100;
  PrevPct = (float) ShowPct / 100.0;
  if ( ShowPct < 1 ) ShowPct = 0;
  if ( ShowPct > 100 ) ShowPct = 100;
  printf( "%3d%% done", ShowPct );
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <stty_raw> <stty_cooked> These procedures change the stty settings    */
/* for stdin to disable or re-enable buffering and echo.                 */
/* They are primarily for use by ReadKey and kbhit routines.             */
/* Can be used to globally disable echo, or re-enable on termination.    */
/*-----------------------------------------------------------------------*/

static struct termio sttyold, sttynew;  /* cooked and raw stty settings */
/* fgetc returns NOKEY if no key pressed and stdin is unbuffered */
#define NOKEY -1
/* PrevKey preserves the keystroke eaten by kbhit */
static char PrevKey = NOKEY;
static int stty_is_raw = 0;  /* current state */

int stty_raw()
{
  int sttyerr = 0;
  int stdin_fnum;

  if ( ! stty_is_raw )
  {
    /* get current settings */
    stdin_fnum = fileno( stdin );
    sttyerr = ioctl( stdin_fnum, TCGETA, &sttyold );
    memmove( &sttynew, &sttyold, sizeof( sttyold ) );
    /* turn off bits to disable line buffering and echo */
    sttynew.c_lflag =
      sttyold.c_lflag & ~( ICANON | ECHO | ECHOE | ECHONL );
    /* clear values to disable character buffering and timeout */
    sttynew.c_cc[ VMIN ] = 0;  sttynew.c_cc[ VTIME ] = 0;
    /* engage new settings */
    if ( ! sttyerr )
      sttyerr = ioctl( stdin_fnum, TCSETA, &sttynew );
    stty_is_raw = ! sttyerr;
    return( sttyerr );
  }
  return( 0 );
}

int stty_cooked()
{
  int sttyerr = 0;
  int stty_mask, stdin_fnum;

  if ( stty_is_raw )
  {
    /* get current settings */
    stdin_fnum = fileno( stdin );
    sttyerr = ioctl( stdin_fnum, TCGETA, &sttynew );
    /* get original values of cleared bits */
    stty_mask =
      sttyold.c_lflag & ( ICANON | ECHO | ECHOE | ECHONL );
    /* clear those bits from current */
    sttynew.c_lflag =
      sttynew.c_lflag & ~( ICANON | ECHO | ECHOE | ECHONL );
    /* set those bits to original values */
    sttynew.c_lflag = sttynew.c_lflag | stty_mask;
    /* set buffering and timeout values to original */
    sttynew.c_cc[ VMIN ] = sttyold.c_cc[ VMIN ];
    sttynew.c_cc[ VTIME ] = sttyold.c_cc[ VTIME ];
    /* engage new settings */
    if ( ! sttyerr )
      sttyerr = ioctl( stdin_fnum, TCSETA, &sttynew );
    stty_is_raw = sttyerr;
    return( sttyerr );
  }
  return( 0 );
}

/*-----------------------------------------------------------------------*/
/* <ReadKey> reads one keypress from stdin and error checks.             */
/*-----------------------------------------------------------------------*/

/* These attributes are set or reset only by the application. */
int READKEY_NOECHO = 0;  /* extern */  /* don't echo keypress */
int READKEY_UC = 0;  /* extern */      /* translate to upper case */

int ReadKey()
{
  int ReadKeyOut;
  int stty_was_raw;

  if ( PrevKey != NOKEY )
  {
    ReadKeyOut = PrevKey;  /* preserved from kbhit() */
    PrevKey = NOKEY;
  }
  else
  {
    stty_was_raw = stty_is_raw;
    stty_raw();
    do
      ReadKeyOut = fgetc( stdin );
    while ( ReadKeyOut == NOKEY );
    if ( ! stty_was_raw )  /* continues prior raw mode */
      stty_cooked();
    if ( ! READKEY_NOECHO && isprint( ReadKeyOut ) )
      putchar( ReadKeyOut );
  }
  if ( READKEY_UC )
    ReadKeyOut = toupper( ReadKeyOut );
  return( ReadKeyOut );
}

/*-----------------------------------------------------------------------*/
/* <kbhit> returns true if a character is waiting in stdin.  Note that   */ 
/* one kbhit uses a destructive technique, so next character must be     */
/* read using ReadKey, not fget* or get*.  ANSI but missing from AT&T.   */
/*-----------------------------------------------------------------------*/

int kbhit()
{
  int KeyPressedOut;
  int stty_was_raw;

  if ( PrevKey != NOKEY )
    KeyPressedOut = PrevKey;
  else
  {
    stty_was_raw = stty_is_raw;
    stty_raw();
    KeyPressedOut = fgetc( stdin );
    if ( KeyPressedOut != NOKEY )
      PrevKey = KeyPressedOut;
    if ( ! stty_was_raw )  /* continues prior raw mode */
      stty_cooked();
  }
  return( KeyPressedOut != NOKEY );
}

/*-----------------------------------------------------------------------*/
/* <EatKeys> consumes all pending keystrokes (stdin buffer).             */
/* It returns the last character read.                                   */
/*-----------------------------------------------------------------------*/

int EatKeys()
{
  int EatKeysOut;

  EatKeysOut = NOKEY;
  while ( kbhit() )
    EatKeysOut = ReadKey();
  return( EatKeysOut );
}

/*-----------------------------------------------------------------------*/
/* <SelectCh> displays a prompt, then waits for a valid keypress.        */
/* Returns key pressed.  Pass null mask to wait for any keypress.        */
/* Returns DfltCh for CR or LF if DfltCh is printable and in mask.       */
/*-----------------------------------------------------------------------*/

/* This attribute is set or reset only by the application. */
int SelectCh_NOECHO = 0;

int SelectCh( Msg, Mask, DfltCh )
char *Msg, *Mask;
int DfltCh;
{
  char SlctCh;
  char *FindCh;
  int PrevUC, PrevECHO;

  printf( "%s > ", Msg );
  DfltCh = toupper( DfltCh );
  if ( ! isprint( DfltCh ) ) DfltCh = '\0';
  if ( isprint( DfltCh ) && ( *Mask ) )   /* verify default in mask */
  {
    FindCh = Mask;
    while ( ( *FindCh ) && ( *FindCh != DfltCh ) ) FindCh++;
    if ( *FindCh != DfltCh ) DfltCh = '\0';
  }
  if ( isprint( DfltCh ) ) printf( "%c", DfltCh );
  PrevECHO = READKEY_NOECHO;  PrevUC = READKEY_UC;
  READKEY_NOECHO = 1;  READKEY_UC = 1;
  do
  {
    SlctCh = ReadKey();
    if ( ( ( SlctCh == 10 ) || ( SlctCh == 13 ) ) &&
         ( isprint( DfltCh ) ) )
      SlctCh = DfltCh;
    FindCh = Mask;
    while ( ( *FindCh ) &&              /* not end of mask */
            ( *FindCh != SlctCh ) )     /* not matching */
      FindCh++;
    if ( ( *Mask ) && ( *FindCh != SlctCh ) )
      Beep();
  } while ( ( *Mask ) && ( *FindCh != SlctCh ) );
  if ( isprint( DfltCh ) ) printf( "\b \b" );
  if ( ( ! SelectCh_NOECHO ) && ( isprint( SlctCh ) ) )
    printf( "%c", SlctCh );
  READKEY_NOECHO = PrevECHO;  READKEY_UC = PrevUC;
  StdErr();
  Escape = Escape || ( SlctCh == 27 );
  return( (char) SlctCh );
}

/*-----------------------------------------------------------------------*/
/* <CheckKey> detects user interruption of process in progress.          */
/* Returns true is escape key has been pressed, pauses for any other.    */
/*-----------------------------------------------------------------------*/

int CheckKey()
{
  char ChkCh;
  int EscPressed, PrevECHO;

  EscPressed = 0;
  if ( ! kbhit() ) return( 0 );
  ChkCh = EatKeys();
  if ( ChkCh == 27 ) /* escape */
  {
    printf( "**** TERMINATED ****" );
    EscPressed = 1;
  }
  else
  {
    printf( "**** PAUSED ****" );
    PrevECHO = READKEY_NOECHO;
    READKEY_NOECHO = 1;
    ChkCh = ReadKey();
    READKEY_NOECHO = PrevECHO;
    printf( "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b" );
    printf( "                " );
    printf( "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b" );
    if ( ChkCh == 27 )
    {
      printf( "**** TERMINATED ****\n" );
      EscPressed = 1;
    }
  }
  EatKeys();
  Escape = Escape || EscPressed;
  return( EscPressed );
}

/*-----------------------------------------------------------------------*/
/* <WaitKey> displays a prompt and waits for any key press.              */
/* Returns true is escape key has been pressed.                          */
/*-----------------------------------------------------------------------*/

int WaitKey()
{
  char PauseKey;
  int PrevECHO;

  printf( "PRESS ANY KEY . . ." );
  PrevECHO = READKEY_NOECHO;
  READKEY_NOECHO = 1;
  PauseKey = ReadKey();  EatKeys;
  READKEY_NOECHO = PrevECHO;
  printf( "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b" );
  printf( "                   " );
  printf( "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b" );
  Escape = Escape || ( PauseKey == 27 );
  return( PauseKey == 27 );
}

/*-----------------------------------------------------------------------*/
/* <Inp_s> is a full featured string data entry routine.                */
/* Returns false if escape pressed.                                      */
/*-----------------------------------------------------------------------*/

#define INP_TEXT 0
#define INP_WORD 1
#define INP_DATE 2
#define INP_TIME 3
#define INP_DOUBLE 4
#define INP_LONG 5
#define INP_FNAME 6

/* InpChange is set if any changes made on most recent input. */
int InpChange = 0;  /* extern */
/* AnyChange is set if InpChange and must be reset by application. */
int AnyChange = 0;  /* extern */

int Inp_s( Value, Len, Typ )
char *Value;
int Len, Typ;
{
  char ReadStr[85], PrevValue[85], ShowStr[85], PrevShowStr[85];
  char InsCh[2];
  char RdCh;
  int CurrLen, CursorPos, PrevCursorPos;
  int InpEscape, Done, NewChar, NoKeyPressed;
  int ShowLp, FirstDif, LastDif;
  int PrevECHO;

  /* initialize */
  InpEscape = 0;  Done = 0;  NoKeyPressed = 1;
  *PrevShowStr = '\0';  PrevCursorPos = 0;  CursorPos = 0;
  if ( Typ == INP_TEXT )
    strcpy( InsCh, " " );
  else
    strcpy( InsCh, "_" );
  /* pass in default */
  strcpy( PrevValue, Value );
  strcpy( ReadStr, Value );
  /* display initial field */
  strcpy( ShowStr, ReadStr );
  PadCh( ShowStr, '_', -Len );
  strcpy( PrevShowStr, ShowStr );
  printf( "%s%s", ShowStr, Rep( '\b', Len ) );
  /* get and process keys */
  PrevECHO = READKEY_NOECHO; 
  READKEY_NOECHO = 1; 
  while ( ( ! Done ) && ( ! InpEscape ) && ( ! _err ) )
  {
    /* refresh display of field */
    strcpy( ShowStr, ReadStr );
    PadCh( ShowStr, '_', -Len );
    /* find first difference since last display */
    FirstDif = -1;  LastDif = -1;
    for ( ShowLp = 0; ShowLp < Len; ShowLp++ )
      if ( ShowStr[ ShowLp ] != PrevShowStr[ ShowLp ] )
      {
        if ( FirstDif == -1 ) FirstDif = ShowLp;
        LastDif = ShowLp;
      }
    /* refresh changed portion */
    if ( FirstDif != -1 )
    {
      /* move cursor to point where changes start */ 
      if ( FirstDif < PrevCursorPos )
        printf( "%s", Rep( '\b', PrevCursorPos - FirstDif ) );
      if ( FirstDif > PrevCursorPos )
        for ( ShowLp = PrevCursorPos; ShowLp != FirstDif; ShowLp++ )
          printf( "%c", ShowStr[ ShowLp ] );
      /* rewrite changed portion */
      for ( ShowLp = FirstDif; ShowLp <= LastDif; ShowLp++ )
        printf( "%c", ShowStr[ ShowLp ] );
      PrevCursorPos = LastDif + 1;
    }
    /* move cursor to new position if changed */
    if ( CursorPos < PrevCursorPos )
      printf( "%s", Rep( '\b', PrevCursorPos - CursorPos ) );
    if ( CursorPos > PrevCursorPos )
      for ( ShowLp = PrevCursorPos; ShowLp != CursorPos; ShowLp++ )
        printf( "%c", ShowStr[ ShowLp ] );
    /* remember current stats */
    PrevCursorPos = CursorPos;  strcpy( PrevShowStr, ShowStr );
    /* done refreshing field, get key */
    RdCh = ReadKey();
#ifdef MSDOS
    if ( Typ == INP_FNAME ) RdCh = toupper( RdCh );
#endif
    if ( ( RdCh >= ' ' ) && ( RdCh <= '~' ) )
    {
      NewChar = 0;
      if ( CursorPos < Len )
      {  /* space for more */
        CurrLen = strlen( ReadStr );
        if ( ( Typ == INP_TEXT ) ||
             ( ( Typ == INP_FNAME ) &&
               ( ! InvalidFileNameChar( RdCh ) ) ) ||
             ( ( Typ == INP_WORD ) && isalnum( RdCh ) ) ||
             ( ( Typ == INP_DATE ) && 
               ( isdigit( RdCh ) ||
                 ( RdCh == '/' ) || ( RdCh == '.' ) ||
                 ( RdCh == '-' ) ) ) ||
             ( ( Typ == INP_DOUBLE ) && 
               ( isdigit( RdCh ) || ( RdCh == '.' ) ||
                 ( toupper( RdCh ) == 'E' ) || ( RdCh == '-' ) ) ) ||
             ( ( Typ == INP_LONG ) && 
               ( isdigit( RdCh ) || ( RdCh == '-' ) ) ) ||
             ( ( Typ == INP_TIME ) && 
               ( isdigit( RdCh ) || ( RdCh == ':' ) ) ) )
        {
          /* erase field if first key pressed is valid character */
          if ( NoKeyPressed )
          {
            *ReadStr = '\0';
            CurrLen = 0;
          }
          if ( CursorPos >= CurrLen )
          {
            if ( ( Typ == INP_DATE ) && ( ! isdigit( RdCh ) ) )
              RdCh = DateSep;
            ReadStr[ CurrLen ] = RdCh;
            ReadStr[ CurrLen + 1 ] = '\0';
            NewChar = 1;
          }
          else
          {
            ReadStr[ CursorPos ] = RdCh;
            NewChar = 1;
          }
          CursorPos++;
        }  /* valid char for type */
      }  /* space for more */
      if ( ! NewChar ) Beep();
    }  /* displayable character */
    else
    switch ( RdCh )
    {
      case 27:  /* escape */
        Done = 0;
        InpEscape = 1;
        break;
      case 23:   /* ^W home */
        if ( ! CursorPos )
          Beep();
        else
          CursorPos = 0;
        break;
      case 20:   /* ^T end */
        if ( CursorPos == strlen( ReadStr ) )
          Beep();
        else
          CursorPos = strlen( ReadStr );
        break;
      case 9:    /* ^I tab */
        if ( CursorPos == strlen( ReadStr ) )
          Beep();
        else
        {
          CursorPos += 8;
          if ( CursorPos > strlen( ReadStr ) )
            CursorPos = strlen( ReadStr );
        }
        break;
      case 18:   /* ^R right */
        if ( CursorPos >= strlen( ReadStr ) )
          Beep();
        else
          CursorPos++;
        break;
      case 5:    /* ^E left */
        if ( ! CursorPos )
          Beep();
        else
          CursorPos--;
        break;
      case 22:   /* ^V ins */
        if ( strlen( ReadStr ) >= Len )
          Beep();
        else
          Insert( InsCh, ReadStr, CursorPos );
        break;
      case 2:   /* ^B del */
        if ( ! strlen( ReadStr ) )
          Beep();
        else
          Delete( ReadStr, CursorPos, 1 );
        break;
      case 8:   /* ^H backspace */
        if ( ( ! CursorPos ) || ( ! strlen( ReadStr ) ) )
          Beep();
        else
        {
          Delete( ReadStr, CursorPos - 1, 1 );
          CursorPos--;
        }
        break;
      case 10:  /* CR or LF done */
      case 13:
        Done = 1;
        break;
      default:
        Beep();
        break;
    }  /* switch */
    NoKeyPressed = 0;
  }  /* while !Done&&!Escape&&!_err */
  /* remove insert characters */
  if ( Typ != INP_TEXT )
  {
    Purge( ReadStr, *InsCh );
    Strip( ReadStr );
  }
  /* retain new input */
  if ( ! InpEscape )
  {
    strcpy( Value, ReadStr );
  }
  strcpy( ShowStr, Value );
  if ( ( Typ == INP_DOUBLE ) || ( Typ == INP_LONG ) )
    Pad( ShowStr, Len );
  else
    Pad( ShowStr, -Len );
  printf( "%s%s", Rep( '\b', CursorPos ), ShowStr );
  InpChange = strcmp( Value, PrevValue );
  AnyChange = AnyChange || InpChange;
  READKEY_NOECHO = PrevECHO; 
  Escape = Escape || InpEscape;
  return( ! InpEscape );
}

/*-----------------------------------------------------------------------*/
/* <Inp_d> accepts a numeric string then validity check for real number. */
/*-----------------------------------------------------------------------*/

int Inp_d( Value, Min, Max )
double *Value;
double Min, Max;
{
  char Work[40], ErrMsg[20];
  double Temp;
  int Len, MinSz, MaxSz;
  int Done = 0;
  int ValOK = 0;

  /* determine maximum field width */
  sprintf( Work, "%lf", Min );
  DeLead( Work, '0' );  DeLead( Work, '.' );
  DeTrail( Work, '0' );  DeTrail( Work, '.' );
  MinSz = strlen( Work );
  sprintf( Work, "%lf", Max );
  DeLead( Work, '0' );  DeLead( Work, '.' );
  DeTrail( Work, '0' );  DeTrail( Work, '.' );
  MaxSz = strlen( Work );
  if ( MinSz > MaxSz )
    Len = MinSz;
  else
    Len = MaxSz;
  do
  {
    sprintf( Work, "%lf", *Value );
    Done = Inp_s( Work, Len, INP_DOUBLE );
    if ( Done )
    {
      if ( ! *Work ) strcpy( Work, "0.0" );
      *ErrMsg = '\0';
      if ( str_d( Work, &Temp ) )
        strcpy( ErrMsg, "BAD INPUT         " );
      else
        if ( ( Temp < Min ) || ( Temp > Max ) )
          strcpy( ErrMsg, "VALUE OUT OF RANGE" );
      if ( *ErrMsg )
      {
        printf( "%s%s", Rep( '\b', Len ), ErrMsg );
        Beep();  Beep();
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
        printf( "%s", Space( strlen( ErrMsg ) ) );
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
      }
      else
      {
        *Value = Temp;
        ValOK = 1;
      }
    }
  } while ( Done && ( ! _err ) && ( ! ValOK ) );
  return( Done );
}

/*-----------------------------------------------------------------------*/
/* <Inp_l> accepts a numeric string then validity check for long.        */
/*-----------------------------------------------------------------------*/

int Inp_l( Value, Min, Max )
long *Value;
long Min, Max;
{
  char Work[40], ErrMsg[20];
  long Temp;
  int Len, MinSz, MaxSz;
  int Done = 0;
  int ValOK = 0;

  /* determine maximum field width */
  sprintf( Work, "%ld", Min );
  DeLead( Work, '0' );  DeLead( Work, '.' );
  DeTrail( Work, '0' );  DeTrail( Work, '.' );
  MinSz = strlen( Work );
  sprintf( Work, "%ld", Max );
  DeLead( Work, '0' );  DeLead( Work, '.' );
  DeTrail( Work, '0' );  DeTrail( Work, '.' );
  MaxSz = strlen( Work );
  if ( MinSz > MaxSz )
    Len = MinSz;
  else
    Len = MaxSz;
  do
  {
    sprintf( Work, "%ld", *Value );
    Done = Inp_s( Work, Len, INP_LONG );
    if ( Done )
    {
      if ( ! *Work ) strcpy( Work, "0" );
      *ErrMsg = '\0';
      if ( str_l( Work, &Temp ) )
        strcpy( ErrMsg, "BAD INPUT         " );
      else
        if ( ( Temp < Min ) || ( Temp > Max ) )
          strcpy( ErrMsg, "VALUE OUT OF RANGE" );
      if ( *ErrMsg )
      {
        printf( "%s%s", Rep( '\b', Len ), ErrMsg );
        Beep();  Beep();
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
        printf( "%s", Space( strlen( ErrMsg ) ) );
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
      }
      else
      {
        *Value = Temp;
        ValOK = 1;
      }
    }
  } while ( Done && ( ! _err ) && ( ! ValOK ) );
  return( Done );
}

/*-----------------------------------------------------------------------*/
/* <Inp_date> accepts a date then validity checks.                       */
/*-----------------------------------------------------------------------*/

int Inp_date( Value )
char *Value;
{
  char Work[9], ErrMsg[20];
  int D_1, D_2, D_3, Yr, Mo, Da;
  char YrStr[3], MoStr[3], DaStr[3];
  int Done = 0;
  int ValOK = 0;

  do
  {
    strcpy( Work, ShowDate( Value, 0 ) );
    Done = Inp_s( Work, 8, INP_DATE );
    if ( Done )
    {
      /* re-arrange based on date format */
      ValOK = ParseDate( Work, &D_1, &D_2, &D_3 );
      if ( ValOK )
      {
        if ( DateFormat == 1 )  /* MDY */
        {
           Yr = D_3;  Mo = D_1;  Da = D_2;
        }
        else
        if ( DateFormat == 2 )  /* DMY */
        {
           Yr = D_3;  Mo = D_2;  Da = D_1;
        }
        else
        {
           Yr = D_1;  Mo = D_2;  Da = D_3;
        }
        strcpy( YrStr, str2( Yr ) );
        strcpy( MoStr, str2( Mo ) );
        strcpy( DaStr, str2( Da ) );
        sprintf( Work, "%s%s%s", YrStr, MoStr, DaStr );
        ValOK = ValidDate( Work );
      }
      if ( ! ValOK )
      {
        strcpy( ErrMsg, "INVALID DATE" );
        printf( "%s%s", Rep( '\b', 8 ), ErrMsg );
        Beep();  Beep();
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
        printf( "%s", Space( strlen( ErrMsg ) ) );
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
      }
      else
      {
        strcpy( Value, Work );
        printf( "%s%s", Rep( '\b', 8 ), ShowDate( Work, 0 ) );
      }
    } 
  } while ( Done && ( ! _err ) && ( ! ValOK ) );
  return( Done );
}

/*-----------------------------------------------------------------------*/
/* <Inp_time> accepts a time then validity checks.                       */
/*-----------------------------------------------------------------------*/

int Inp_time( Value )
char *Value;
{
  char Work[9], ErrMsg[20];
  int Done = 0;
  int ValOK = 0;
  int Hr, Mn, Sc;

  do
  {
    strcpy( Work, Value );
    Work[ 5 ] = '\0';
    Done = Inp_s( Work, 5, INP_TIME );
    if ( Done )
    {
      ValOK = ValidTime( Work );
      if ( ! ValOK )
      {
        strcpy( ErrMsg, "INVALID TIME" );
        printf( "%s%s", Rep( '\b', 5 ), ErrMsg );
        Beep();  Beep();
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
        printf( "%s", Space( strlen( ErrMsg ) ) );
        printf( "%s", Rep( '\b', strlen( ErrMsg ) ) );
      }
      else
      {
        ParseTime( Work, &Hr, &Mn, &Sc );
        strcpy( Work, str2( Mn ) );
        sprintf( Value, "%s:%s", str2( Hr ), Work );
        printf( "%s%s", Rep( '\b', 5 ), Value );
      }
    } 
  } while ( Done && ( ! _err ) && ( ! ValOK ) );
  return( Done );
}
